from abc import ABC, abstractmethod
from contextlib import closing
from os import getenv

import MySQLdb
import pandas as pd
import pandas_gbq

from . import consts


class Source(ABC):
    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def query(self, query: str, output_type: str = "pandas"):
        pass


class MySqlSource(Source):
    def __init__(self, database):
        super().__init__()
        
        self.host = getenv(f"MYSQL_{database}_HOST".upper())
        self.port = int(getenv(f"MYSQL_{database}_PORT".upper()))
        self.user = getenv(f"MYSQL_{database}_USER".upper())
        self.password = getenv(f"MYSQL_{database}_PASS".upper())
        self.db = database

    def query(self, query: str, output_type: str = "pandas"):
        connection = MySQLdb.connect(self.host, self.user, self.password, self.db, self.port, charset="utf8")
        return self._query_mysql(query, connection, output_type)

    @staticmethod
    def _query_mysql(
            query: str, connection: MySQLdb.Connection, output_type: str = "pandas"
    ) -> pd.DataFrame:
        cursor = connection.cursor()
        with closing(cursor):
            if output_type == "pandas":
                return pd.read_sql(query, con=connection)
            else:
                raise NotImplementedError


class GoogleBigQuerySource(Source):
    def __init__(self, dataset: str):
        super().__init__()

        self.project_id = getenv(f"GBQ_{dataset}_PROJECT_ID".upper(), "")

    def query(self, query: str, output_type: str = "pandas"):
        return self._query_gbq(query, self.project_id, output_type)

    @staticmethod
    def _query_gbq(
        query: str, project_id: str, output_type: str = "pandas"
    ) -> pd.DataFrame:
        if output_type == "pandas":
            return pandas_gbq.read_gbq(query=query, project_id=project_id)
        else:
            raise NotImplementedError


ENGINES = {
    consts.engines.MYSQL: MySqlSource
}



from pathlib import Path

from dotenv import load_dotenv

from . import sources

env_path = Path(".") / ".env"
load_dotenv(dotenv_path=env_path)


def source(engine, **kwargs):
    try:
        src = sources.ENGINES[engine]
    except KeyError:
        raise NotImplementedError(f"{engine} engine is not supported")

    return src(**kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['datatosk', 'datatosk.consts']

package_data = \
{'': ['*']}

install_requires = \
['bottleneck>=1.3.2,<2.0.0',
 'google-cloud-bigquery>=1.24.0,<2.0.0',
 'mysqlclient>=1.4.6,<2.0.0',
 'numexpr>=2.7.1,<3.0.0',
 'pandas-gbq>=0.13.1,<0.14.0',
 'pandas>=1.0.1,<2.0.0',
 'python-dotenv>=0.12.0,<0.13.0',
 'sqlalchemy>=1.3.13,<2.0.0']

setup_kwargs = {
    'name': 'datatosk',
    'version': '0.1.3',
    'description': '',
    'long_description': None,
    'author': 'Miłosz Bednarzak',
    'author_email': 'milosz.bednarzak@bethink.pl',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
